# Copyright (C) 2019 Alteryx, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Exception definitions for ayx-learn."""


class DataScienceException(Exception):
    """Base exception for data science objects."""

    pass


class OheUnexpectedLevelsException(DataScienceException):
    """Exception for unexpected level in One Hot Encoding."""

    def __init__(self, message, errors):
        self.levels, self.column_name = errors
        super().__init__(message)


class NullValueError(DataScienceException):
    """Exception for a dataframe containing null values."""

    def __init__(self, message, column_name):
        self.column_name = column_name
        super().__init__(message)
